#include "stdafx.h"
#include "DIBFile.h"

CDIBFile::CDIBFile ()
{
	m_hFile = INVALID_HANDLE_VALUE;
	m_hFileMap = NULL;
	m_hFileMapView = NULL;
}

CDIBFile::~CDIBFile ()
{
	Clean ();
}

void CDIBFile::Clean (void)
{
	if (m_hFileMapView != NULL) {
		VERIFY (UnmapViewOfFile (m_hFileMapView));
		m_hFileMapView = NULL;
	}
	if (m_hFileMap != NULL) {
		VERIFY (CloseHandle (m_hFileMap));
		m_hFileMap = NULL;
	}
	if (m_hFile != INVALID_HANDLE_VALUE) {
		VERIFY (CloseHandle (m_hFile));
		m_hFile = INVALID_HANDLE_VALUE;
	}
}

BGR* CDIBFile::Load (LPCTSTR szName)
{
	Clean ();

	m_hFile = CreateFile (szName, GENERIC_READ, FILE_SHARE_READ, NULL,
		OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (m_hFile == INVALID_HANDLE_VALUE)
		return NULL;

	m_hFileMap = CreateFileMapping (m_hFile, NULL, PAGE_READONLY | SEC_COMMIT,
		0, 0, NULL); 
	if (m_hFileMap == NULL) {
		Clean ();
		return NULL;
	}

	m_hFileMapView = MapViewOfFile (m_hFileMap, FILE_MAP_READ, 0, 0, 0);
	if (m_hFileMapView == NULL) {
		Clean ();
		return NULL;
	}

	BITMAPFILEHEADER *bfh = (BITMAPFILEHEADER*) m_hFileMapView;
	ASSERT (bfh->bfType == 0x4d42);		// 'BM'
	ASSERT (bfh->bfReserved1 == 0);
	ASSERT (bfh->bfReserved2 == 0);
	BITMAPINFOHEADER *bih = (BITMAPINFOHEADER*) ((char*) m_hFileMapView +
		sizeof (BITMAPFILEHEADER));
	ASSERT (bih->biSize == sizeof (BITMAPINFOHEADER));
	ASSERT (bih->biPlanes == 1);		// one plane
	ASSERT (bih->biBitCount == 24);		// true-color
	ASSERT (bih->biCompression == 0);	// not compressed
	BGR *bits =  (BGR*) ((char*) m_hFileMapView + bfh->bfOffBits);

	return bits;
}
